@@ -30,8 +30,6 @@
 #include <syslog.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
-
-#include <alsa/asoundlib.h>
 #include "alsactl.h"
 
 int file_map(const char *filename, char **buf, size_t *bufsize)
@@ -92,12 +90,12 @@ void initfailed(int cardnumber, const char *reason, int exitcode)
 		return;
 	sprintf(sexitcode, "%i", exitcode);
 	fp = open(statefile, O_WRONLY|O_CREAT|O_APPEND, 0644);
-	write(fp, str, strlen(str));
-	write(fp, ":", 1);
-	write(fp, reason, strlen(reason));
-	write(fp, ":", 1);
-	write(fp, sexitcode, strlen(sexitcode));
-	write(fp, "\n", 1);
+	(void)write(fp, str, strlen(str));
+	(void)write(fp, ":", 1);
+	(void)write(fp, reason, strlen(reason));
+	(void)write(fp, ":", 1);
+	(void)write(fp, sexitcode, strlen(sexitcode));
+	(void)write(fp, "\n", 1);
 	close(fp);
 	free(str);
 }
@@ -193,3 +191,139 @@ void error_handler(const char *file, int line, const char *function, int err, co
 		fprintf(stderr, "alsa-lib %s:%i:(%s) %s%s%s\n", file, line, function,
 				buf, err ? ": " : "", err ? snd_strerror(err) : "");
 }
+
+int load_configuration(const char *file, snd_config_t **top, int *open_failed)
+{
+	snd_config_t *config;
+	snd_input_t *in;
+	int err, stdio_flag, lock_fd = -EINVAL;
+
+	*top = NULL;
+	if (open_failed)
+		*open_failed = 0;
+	err = snd_config_top(&config);
+	if (err < 0) {
+		error("snd_config_top error: %s", snd_strerror(err));
+		return err;
+	}
+	stdio_flag = !strcmp(file, "-");
+	if (stdio_flag) {
+		err = snd_input_stdio_attach(&in, stdin, 0);
+	} else {
+		lock_fd = state_lock(file, 10);
+		err = lock_fd >= 0 ? snd_input_stdio_open(&in, file, "r") : lock_fd;
+	}
+	if (err < 0) {
+		if (open_failed)
+			*open_failed = 1;
+		goto out;
+	}
+	err = snd_config_load(config, in);
+	snd_input_close(in);
+	if (lock_fd >= 0)
+		state_unlock(lock_fd, file);
+	if (err < 0) {
+		error("snd_config_load error: %s", snd_strerror(err));
+out:
+		snd_config_delete(config);
+		snd_config_update_free_global();
+		return err;
+	} else {
+		*top = config;
+		return 0;
+	}
+}
+
+void snd_card_iterator_init(struct snd_card_iterator *iter, int cardno)
+{
+	iter->card = cardno;
+	iter->single = cardno >= 0;
+	iter->first = true;
+	iter->name[0] = '\0';
+}
+
+int snd_card_iterator_sinit(struct snd_card_iterator *iter, const char *cardname)
+{
+	int cardno = -1;
+
+	if (cardname) {
+		cardno = snd_card_get_index(cardname);
+		if (cardno < 0) {
+			error("Cannot find soundcard '%s'...", cardname);
+			return cardno;
+		}
+	}
+	snd_card_iterator_init(iter, cardno);
+	return 0;
+}
+
+const char *snd_card_iterator_next(struct snd_card_iterator *iter)
+{
+	if (iter->single) {
+		if (iter->first) {
+			iter->first = false;
+			goto retval;
+		}
+		return NULL;
+	}
+	if (snd_card_next(&iter->card) < 0) {
+		if (!ignore_nocards && iter->first)
+			error("No soundcards found...");
+		return NULL;
+	}
+	iter->first = false;
+	if (iter->card < 0)
+		return NULL;
+retval:
+	snprintf(iter->name, sizeof(iter->name), "hw:%d", iter->card);
+
+	return (const char *)iter->name;
+}
+
+int snd_card_iterator_error(struct snd_card_iterator *iter)
+{
+	return iter->first ? (ignore_nocards ? 0 : -ENODEV) : 0;
+}
+
+static int cleanup_filename_filter(const struct dirent *dirent)
+{
+	size_t flen;
+
+	if (dirent == NULL)
+		return 0;
+	if (dirent->d_type == DT_DIR)
+		return 0;
+
+	flen = strlen(dirent->d_name);
+	if (flen <= 5)
+		return 0;
+
+	if (strncmp(&dirent->d_name[flen-5], ".conf", 5) == 0)
+		return 1;
+
+	return 0;
+}
+
+int snd_card_clean_cfgdir(const char *cfgdir, int cardno)
+{
+	char path[PATH_MAX];
+	struct dirent **list;
+	int lasterr = 0, n, j;
+
+	snprintf(path, sizeof(path), "%s/card%d.conf.d", cfgdir, cardno);
+	n = scandir(path, &list, cleanup_filename_filter, NULL);
+	if (n < 0) {
+		if (errno == ENOENT)
+			return 0;
+		return -errno;
+	}
+	for (j = 0; j < n; j++) {
+		snprintf(path, sizeof(path), "%s/card%d.conf.d/%s", cfgdir, cardno, list[j]->d_name);
+		if (remove(path)) {
+			error("Unable to remove file '%s'", path);
+			lasterr = -errno;
+		}
+	}
+
+	return lasterr;
+}
