@@ -32,6 +32,7 @@
 #include <malloc.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
 #include <getopt.h>
 #include <fcntl.h>
@@ -94,6 +95,7 @@ enum {
 	VUMETER_STEREO
 };
 
+static snd_pcm_format_t default_format = DEFAULT_FORMAT;
 static char *command;
 static snd_pcm_t *handle;
 static struct {
@@ -468,6 +470,24 @@ static long parse_long(const char *str, int *err)
 	return val;
 }
 
+static void try_to_adjust_default_format_16bit(void)
+{
+	snd_pcm_hw_params_t *params;
+	int err;
+
+	snd_pcm_hw_params_alloca(&params);
+	err = snd_pcm_hw_params_any(handle, params);
+	if (err < 0) {
+		error(_("Broken configuration for this PCM: no configurations available"));
+		prg_exit(EXIT_FAILURE);
+	}
+
+	if (file_type != FORMAT_AU && snd_pcm_hw_params_test_format(handle, params, SND_PCM_FORMAT_S16_LE) == 0)
+		rhwparams.format = default_format = SND_PCM_FORMAT_S16_LE;
+	else if (file_type == FORMAT_AU && snd_pcm_hw_params_test_format(handle, params, SND_PCM_FORMAT_S16_BE) == 0)
+		rhwparams.format = default_format = SND_PCM_FORMAT_S16_BE;
+}
+
 int main(int argc, char *argv[])
 {
 	int duration_or_sample = 0;
@@ -528,6 +548,7 @@ int main(int argc, char *argv[])
 	int do_device_list = 0, do_pcm_list = 0;
 	snd_pcm_info_t *info;
 	FILE *direction;
+	bool user_set_fmt = false;
 
 #ifdef ENABLE_NLS
 	setlocale(LC_ALL, "");
@@ -562,7 +583,7 @@ int main(int argc, char *argv[])
 	}
 
 	chunk_size = -1;
-	rhwparams.format = DEFAULT_FORMAT;
+	rhwparams.format = default_format;
 	rhwparams.rate = DEFAULT_SPEED;
 	rhwparams.channels = 1;
 
@@ -612,6 +633,7 @@ int main(int argc, char *argv[])
 			}
 			break;
 		case 'f':
+			user_set_fmt = true;
 			if (strcasecmp(optarg, "cd") == 0 || strcasecmp(optarg, "cdr") == 0) {
 				if (strcasecmp(optarg, "cdr") == 0)
 					rhwparams.format = SND_PCM_FORMAT_S16_BE;
@@ -844,6 +866,9 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if (!user_set_fmt)
+		try_to_adjust_default_format_16bit();
+
 	chunk_size = 1024;
 	hwparams = rhwparams;
 
@@ -1064,7 +1089,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 	hwparams.channels = channels;
 	switch (TO_CPU_SHORT(f->bit_p_spl, big_endian)) {
 	case 8:
-		if (hwparams.format != DEFAULT_FORMAT &&
+		if (hwparams.format != default_format &&
 		    hwparams.format != SND_PCM_FORMAT_U8)
 			fprintf(stderr, _("Warning: format is changed to U8\n"));
 		hwparams.format = SND_PCM_FORMAT_U8;
@@ -1074,7 +1099,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 			native_format = SND_PCM_FORMAT_S16_BE;
 		else
 			native_format = SND_PCM_FORMAT_S16_LE;
-		if (hwparams.format != DEFAULT_FORMAT &&
+		if (hwparams.format != default_format &&
 		    hwparams.format != native_format)
 			fprintf(stderr, _("Warning: format is changed to %s\n"),
 				snd_pcm_format_name(native_format));
@@ -1087,7 +1112,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 				native_format = SND_PCM_FORMAT_S24_3BE;
 			else
 				native_format = SND_PCM_FORMAT_S24_3LE;
-			if (hwparams.format != DEFAULT_FORMAT &&
+			if (hwparams.format != default_format &&
 			    hwparams.format != native_format)
 				fprintf(stderr, _("Warning: format is changed to %s\n"),
 					snd_pcm_format_name(native_format));
@@ -1098,7 +1123,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 				native_format = SND_PCM_FORMAT_S24_BE;
 			else
 				native_format = SND_PCM_FORMAT_S24_LE;
-			if (hwparams.format != DEFAULT_FORMAT &&
+			if (hwparams.format != default_format &&
 			    hwparams.format != native_format)
 				fprintf(stderr, _("Warning: format is changed to %s\n"),
 					snd_pcm_format_name(native_format));
@@ -1184,19 +1209,19 @@ static int test_au(int fd, void *buffer)
 	pbrec_count = BE_INT(ap->data_size);
 	switch (BE_INT(ap->encoding)) {
 	case AU_FMT_ULAW:
-		if (hwparams.format != DEFAULT_FORMAT &&
+		if (hwparams.format != default_format &&
 		    hwparams.format != SND_PCM_FORMAT_MU_LAW)
 			fprintf(stderr, _("Warning: format is changed to MU_LAW\n"));
 		hwparams.format = SND_PCM_FORMAT_MU_LAW;
 		break;
 	case AU_FMT_LIN8:
-		if (hwparams.format != DEFAULT_FORMAT &&
+		if (hwparams.format != default_format &&
 		    hwparams.format != SND_PCM_FORMAT_U8)
 			fprintf(stderr, _("Warning: format is changed to U8\n"));
 		hwparams.format = SND_PCM_FORMAT_U8;
 		break;
 	case AU_FMT_LIN16:
-		if (hwparams.format != DEFAULT_FORMAT &&
+		if (hwparams.format != default_format &&
 		    hwparams.format != SND_PCM_FORMAT_S16_BE)
 			fprintf(stderr, _("Warning: format is changed to S16_BE\n"));
 		hwparams.format = SND_PCM_FORMAT_S16_BE;
@@ -1528,6 +1553,19 @@ static void done_stdin(void)
 	tcsetattr(fileno(stdin), TCSANOW, &term);
 }
 
+static char wait_for_input(void)
+{
+	struct pollfd pfd;
+	unsigned char b;
+
+	do {
+		pfd.fd = fileno(stdin);
+		pfd.events = POLLIN;
+		poll(&pfd, 1, -1);
+	} while (read(fileno(stdin), &b, 1) != 1);
+	return b;
+}
+
 static void do_pause(void)
 {
 	int err;
@@ -1546,7 +1584,7 @@ static void do_pause(void)
 		return;
 	}
 	while (1) {
-		while (read(fileno(stdin), &b, 1) != 1);
+		b = wait_for_input();
 		if (b == ' ' || b == '\r') {
 			while (read(fileno(stdin), &b, 1) == 1);
 			if (snd_pcm_state(handle) == SND_PCM_STATE_SUSPENDED)
@@ -1571,7 +1609,7 @@ static void check_stdin(void)
 				while (read(fileno(stdin), &b, 1) == 1);
 				fprintf(stderr, _("\r=== PAUSE ===                                                            "));
 				fflush(stderr);
-			do_pause();
+				do_pause();
 				fprintf(stderr, "                                                                          \r");
 				fflush(stderr);
 			}
@@ -1651,7 +1689,8 @@ static void xrun(void)
 			prg_exit(EXIT_FAILURE);
 		}
 		return;		/* ok, data should be accepted again */
-	} if (snd_pcm_status_get_state(status) == SND_PCM_STATE_DRAINING) {
+	}
+	if (snd_pcm_status_get_state(status) == SND_PCM_STATE_DRAINING) {
 		if (verbose) {
 			fprintf(stderr, _("Status(DRAINING):\n"));
 			snd_pcm_status_dump(status, log);
@@ -1914,22 +1953,46 @@ static void do_test_position(void)
 	static snd_pcm_sframes_t minavail, mindelay;
 	static snd_pcm_sframes_t badavail = 0, baddelay = 0;
 	snd_pcm_sframes_t outofrange;
-	snd_pcm_sframes_t avail, delay;
+	snd_pcm_sframes_t avail, delay, savail, sdelay;
+	snd_pcm_status_t *status;
 	int err;
 
+	snd_pcm_status_alloca(&status);
 	err = snd_pcm_avail_delay(handle, &avail, &delay);
 	if (err < 0)
 		return;
+	err = snd_pcm_status(handle, status);
+	if (err < 0)
+		return;
+	savail = snd_pcm_status_get_avail(status);
+	sdelay = snd_pcm_status_get_delay(status);
 	outofrange = (test_coef * (snd_pcm_sframes_t)buffer_frames) / 2;
 	if (avail > outofrange || avail < -outofrange ||
 	    delay > outofrange || delay < -outofrange) {
-	  badavail = avail; baddelay = delay;
-	  availsum = delaysum = samples = 0;
-	  maxavail = maxdelay = 0;
-	  minavail = mindelay = buffer_frames * 16;
-	  fprintf(stderr, _("Suspicious buffer position (%li total): "
-	  	"avail = %li, delay = %li, buffer = %li\n"),
-	  	++counter, (long)avail, (long)delay, (long)buffer_frames);
+		badavail = avail; baddelay = delay;
+		availsum = delaysum = samples = 0;
+		maxavail = maxdelay = 0;
+		minavail = mindelay = buffer_frames * 16;
+		fprintf(stderr, _("Suspicious buffer position (%li total): "
+			"avail = %li, delay = %li, buffer = %li\n"),
+			++counter, (long)avail, (long)delay, (long)buffer_frames);
+	} else if (savail > outofrange || savail < -outofrange ||
+		   sdelay > outofrange || sdelay < -outofrange) {
+		badavail = savail; baddelay = sdelay;
+		availsum = delaysum = samples = 0;
+		maxavail = maxdelay = 0;
+		minavail = mindelay = buffer_frames * 16;
+		fprintf(stderr, _("Suspicious status buffer position (%li total): "
+			"avail = %li, delay = %li, buffer = %li\n"),
+			++counter, (long)savail, (long)sdelay, (long)buffer_frames);
+	} else if (stream == SND_PCM_STREAM_CAPTURE && avail > delay) {
+		fprintf(stderr, _("Suspicious buffer position avail > delay (%li total): "
+			"avail = %li, delay = %li\n"),
+			++counter, (long)avail, (long)delay);
+	} else if (stream == SND_PCM_STREAM_CAPTURE && savail > sdelay) {
+		fprintf(stderr, _("Suspicious status buffer position avail > delay (%li total): "
+			"avail = %li, delay = %li\n"),
+			++counter, (long)savail, (long)sdelay);
 	} else if (verbose) {
 		time(&now);
 		if (tmr == (time_t) -1) {
@@ -1940,19 +2003,27 @@ static void do_test_position(void)
 		}
 		if (avail > maxavail)
 			maxavail = avail;
+		if (savail > maxavail)
+			maxavail = savail;
 		if (delay > maxdelay)
 			maxdelay = delay;
+		if (sdelay > maxdelay)
+			maxdelay = sdelay;
 		if (avail < minavail)
 			minavail = avail;
+		if (savail < minavail)
+			minavail = savail;
 		if (delay < mindelay)
 			mindelay = delay;
+		if (sdelay < mindelay)
+			mindelay = sdelay;
 		availsum += avail;
 		delaysum += delay;
 		samples++;
-		if (avail != 0 && now != tmr) {
+		if ((maxavail != 0 || maxdelay != 0) && now != tmr) {
 			fprintf(stderr, "BUFPOS: avg%li/%li "
 				"min%li/%li max%li/%li (%li) (%li:%li/%li)\n",
-				(long)(availsum / samples),
+                         (long)(availsum / samples),
 				(long)(delaysum / samples),
 				(long)minavail, (long)mindelay,
 				(long)maxavail, (long)maxdelay,
@@ -1961,6 +2032,10 @@ static void do_test_position(void)
 			tmr = now;
 		}
 	}
+	if (verbose == 1) {
+		fprintf(stderr, _("Status(R/W) (standalone avail=%li delay=%li):\n"), (long)avail, (long)delay);
+		snd_pcm_status_dump(status, log);
+	}
 }
 
 /*
@@ -2315,7 +2390,7 @@ static void voc_play(int fd, int ofs, char *name)
 			prg_exit(EXIT_FAILURE);
 		}
 	}
-	hwparams.format = DEFAULT_FORMAT;
+	hwparams.format = default_format;
 	hwparams.channels = 1;
 	hwparams.rate = DEFAULT_SPEED;
 	set_params();
@@ -2495,7 +2570,9 @@ static void voc_play(int fd, int ofs, char *name)
 		}
 	}			/* while(1) */
       __end:
-        voc_pcm_flush();
+	if (!in_aborting) {
+		voc_pcm_flush();
+	}
         free(buf);
 }
 /* that was a big one, perhaps somebody split it :-) */
@@ -2810,9 +2887,11 @@ static void playback_go(int fd, size_t loaded, off64_t count, int rtype, char *n
 		written += r;
 		l = 0;
 	}
-	snd_pcm_nonblock(handle, 0);
-	snd_pcm_drain(handle);
-	snd_pcm_nonblock(handle, nonblock);
+	if (!in_aborting) {
+		snd_pcm_nonblock(handle, 0);
+		snd_pcm_drain(handle);
+		snd_pcm_nonblock(handle, nonblock);
+	}
 }
 
 static int read_header(int *loaded, int header_size)
@@ -3268,7 +3347,7 @@ static void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t
 		do {
 			r = safe_read(fds[0], bufs[0], expected);
 			if (r < 0) {
-				perror(names[channel]);
+				perror(names[0]);
 				prg_exit(EXIT_FAILURE);
 			}
 			for (channel = 1; channel < channels; ++channel) {
@@ -3288,9 +3367,11 @@ static void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t
 		r = r * bits_per_frame / 8;
 		count -= r;
 	}
-	snd_pcm_nonblock(handle, 0);
-	snd_pcm_drain(handle);
-	snd_pcm_nonblock(handle, nonblock);
+	if (!in_aborting) {
+		snd_pcm_nonblock(handle, 0);
+		snd_pcm_drain(handle);
+		snd_pcm_nonblock(handle, nonblock);
+	}
 }
 
 static void capturev_go(int* fds, unsigned int channels, off64_t count, int rtype, char **names)
